( * 3iBeats.fs - AE )

\ include 3iShift.fs  (74165 shift in register IC for inputting beats from DIP switches)
\ include 3iSensor.fs (4x4 matrix keypad for inputting beats from keypad)



\ APPLICATION
variable beatdur
: beatdur! ( dur -- ) beatdur ! ;
: beatdur@ ( -- dur ) beatdur @ ;  \ TODO: have this be read by input POTENTIOMETER for rate control

: init-beats
  buzzer out
  100 beatdur!  \ default - 16th note
; 


: // beatdur@ ms ;  \ Timings from Forth, 100ms = 16th note, 200ms = 8th note, 400ms = quarter note, and 800ms = half note, 1600ms = whole note
: metronome  begin // buzzer pulse-hi  key? until ;  \ metronome
: Nmetronome ( dur -- ) >r begin r@ ms buzzer pulse-hi key? until r> drop ; 

\ beats: 237, 118, 17, 13, 213, 253, 0
\ seqs: 0 213 13 17    and   1 253 13 17 0 213 13 17
: beat-byte  ( b -- ) 8 0 do dup 1 and if buzzer pulse-hi then // 1 rshift loop drop ;  \ parse byte LSB first to beat
: Nbeat ( N-bytes N -- ) 0 do beat-byte loop ;  \  237 118  2 Nbeat
: Nseq ( N-bytes N -- ) >r begin r@ ndup r@ nbeat key? until r> ndrop ;
: Nsamplebeat  ( N -- ) >r r@ Nsample r> Nseq ;
: demo-uptown-funk 1 253 13 17 0 213 13 17   8 Nseq ;  \ 16 notes = 4 bars uptown funky bunch: https://www.youtube.com/watch?v=M1F0lBnsnkE

\ APPLICATION IDEA: Use a single 8-pin DIP to create 2 note combinations, and OLED spits out their binary encoding, 
\    touchpad entry allows typing them in, and uC accepts them as the basis for looping
\ THIS IS WHERE SOFTWARE BEATS HARDWARE!  Ultimately, we need the hardware for calculation purposes.  The experiment is software
\ And if the interface were virtual, then would not need the hardware at all.  The hardware is useful when you don't want a computer
\ And for precise timing independent of Microsoft Windows


\  KEYPAD TEST -- include 3iSensor.fs

17 constant buzzer
init-beats
: beep ( -- ) buzzer hi 5 ms buzzer lo 50 ms ;
: buzzcount ( N -- ) 0 do beep loop ;
: keypad-demo ( -- ) 
    begin  fastkeypad 1 + buzzcount  key?  until  ;

create beatspad 0 c, 1 c, 0 c, 0 c, 0 c, 0 c, 0 c, 0 c, 0 c, 0 c, 0 c, 0 c, 0 c, 0 c, 0 c, 0 c, 
        
\ APPLICATION - use 74165 to allow a simple ATTINY to monitor a keypad continuously and send out the results, so not to slow down the main uC.  Another application for the GA144...

: drawnote ( ascii -- ) dup 1 = if 14 else 22 then emit space ;
: nibblespace? ( -- ) 1 + 4 mod 0 = if 2 spaces then ;
: beats-demo ( -- )
cr ." Enter 2 bars of 8x 16th-note beats (press note or rest): " cr
    8 0 do fastkeypad bytes beatspad + c@ drawnote i nibblespace? loop >b8 1 Nseq  \ get 8 entries
; 


\ TODO: Expand to a menu system:  ADD bar, EDIT, DELETE bar.  
\       Extract from keypad to fewest number of buttons
\       Add potentiometer for rate control


\ ======================================================================

: help-beats
  cr ." include   3iForth.fs  3iShift.fs  3iBeats.fs   (requires Nano running 3iForth.avr)"
  cr ." hardware: requires Nano, either 74165 IC on nano pins 8,9,10, 8-bit dip or switches, buzzer on nano pin 4"
  cr ."         OR  Nano, 4x4 matrix keypad on nano pins 2-9, buzzer on nano pin 17"
  cr ." 1 open-com   rx"
  cr ." init-shiftin  OR  init-beats"
  cr ." 200 Nmetronome (quarter notes)  metronome (16th notes)"
  cr ." latch-in [sample]  or  N Nsample   N Nsamplebeat"
  cr ." keypad-init   create keypad (16 values)"
  cr ." fastkeypad   fastkeypad-test"
  cr ." N Nbeat  N Nseq"
  cr ." demo-uptown-funk"
  cr ." beats-demo"
;


\ V2 - 2021-05-26---Wed---07:52  extracted from Shift.fs.  Added keypad entry with software prompt for 2 bars